use std::rc::Rc;
use std::cell::RefCell;

/// [208.实现 Trie (前缀树)](https://leetcode-cn.com/problems/implement-trie-prefix-tree/)
///
/// [剑指 Offer II 062. 实现前缀树](https://leetcode.cn/problems/QC3q1f/)
///
/// 2021-04-14 20:20:51
///
/// Trie（发音类似 "try"）或者说 前缀树 是一种树形数据结构，用于高效地存储和检索字符串数据集中的键。这一数据结构有相当多的应用情景，例如自动补完和拼
/// 写检查。
///
/// 请你实现 Trie 类：
///
/// Trie() 初始化前缀树对象。
///
/// void insert(String word) 向前缀树中插入字符串 word 。
///
/// boolean search(String word) 如果字符串 word 在前缀树中，返回 true（即，在检索之前已经插入）；否则，返回 false。
/// boolean startsWith(String prefix) 如果之前已经插入的字符串 word 的前缀之一为 prefix ，返回 true ；否
/// 则，返回 false 。
///
/// 示例：
///
///     输入
///         `["Trie", "insert", "search", "search", "startsWith", "insert", "search"]`
///         `[[], ["apple"], ["apple"], ["app"], ["app"], ["app"], ["app"]]`
///     输出
///         `[null, null, true, false, true, null, true]`
///
/// 解释
/// ```java
/// Trie trie = new Trie();
/// trie.insert("apple");
/// trie.search("apple");   // 返回 True
/// trie.search("app");     // 返回 False
/// trie.startsWith("app"); // 返回 True
/// trie.insert("app");
/// trie.search("app");     // 返回 True
/// ```
///
/// 提示：
/// - 1 <= word.length, prefix.length <= 2000
/// - word 和 prefix 仅由小写英文字母组成
/// - insert、search 和 startsWith 调用次数 总计 不超过 3 * 104 次
///
/// Related Topics 设计 字典树
/// 👍 704 👎 0
#[derive(Default)]
struct Trie([Option<Box<Trie>>; 26], bool);

impl Trie {
    fn new() -> Self {
        Default::default()
    }

    fn insert(&mut self, word: String) {
        word.into_bytes().into_iter().map(|c| (c - b'a') as usize).fold(self, |t, i| t.0[i].get_or_insert_with(Default::default)).1 = true
    }

    fn find(&self, prefix: String) -> (bool, bool) {
        let mut t = self;
        for i in prefix.into_bytes().into_iter().map(|c| (c - b'a') as usize) {
            match &t.0[i] {
                Some(s) => t = &s,
                _ => return (false, false)
            }
        }
        (true, t.1)
    }

    fn search(&self, word: String) -> bool {
        self.find(word).1
    }

    fn starts_with(&self, prefix: String) -> bool {
        self.find(prefix).0
    }
}


#[cfg(test)]
mod test {
    #[test]
    fn test_q0208() {
        let mut trie = crate::design::q0208::Trie::new();

        trie.insert("apple".to_string());

        let b1 = trie.search("apple".to_string());
        assert_eq!(b1, true);

        let b2 = trie.search("app".to_string());
        assert_eq!(b2, false);

        let b1 = trie.starts_with("app".to_string());
        assert_eq!(b1, true);

        trie.insert("app".to_string());

        let b1 = trie.search("app".to_string());
        assert_eq!(b1, true);
    }
}
