use crate::Solution;

impl Solution {
    /// 13.罗马数字转整数
    ///
    /// [原题链接](https://leetcode-cn.com/problems/roman-to-integer/)
    ///
    /// 2021-05-15 10:21:35
    ///
    /// 罗马数字包含以下七种字符: I， V， X， L，C，D 和 M。
    ///
    /// |字符           |数值  |
    /// |--------------|-----|
    /// |I             |1    |
    /// |V             |5    |
    /// |X             |10   |
    /// |L             |50   |
    /// |C             |100  |
    /// |D             |500  |
    /// |M             |1000 |
    ///
    /// 例如， 罗马数字 2 写做 II ，即为两个并列的 1。12 写做 XII ，即为 X + II 。 27 写做 XXVII, 即为 XX + V + II 。
    ///
    /// 通常情况下，罗马数字中小的数字在大的数字的右边。但也存在特例，例如 4 不写做 IIII，而是 IV。数字 1 在数字 5 的左边，所表示的数等于大数 5
    /// 减小数 1 得到的数值 4 。同样地，数字 9 表示为 IX。这个特殊的规则只适用于以下六种情况：
    ///
    ///
    /// + I 可以放在 V (5) 和 X (10) 的左边，来表示 4 和 9。
    /// + X 可以放在 L (50) 和 C (100) 的左边，来表示 40 和 90。
    /// + C 可以放在 D (500) 和 M (1000) 的左边，来表示 400 和 900。
    ///
    ///
    /// 给定一个罗马数字，将其转换成整数。输入确保在 1 到 3999 的范围内。
    ///
    ///
    ///
    /// + 示例 1:
    ///     - 输入: "III"
    ///     - 输出: 3
    /// + 示例 2:
    ///     - 输入: "IV"
    ///     - 输出: 4
    /// + 示例 3:
    ///     - 输入: "IX"
    ///     - 输出: 9
    /// + 示例 4:
    ///     - 输入: "LVIII"
    ///     - 输出: 58
    ///     - 解释: L = 50, V= 5, III = 3.
    /// + 示例 5:
    ///     - 输入: "MCMXCIV"
    ///     - 输出: 1994
    ///     - 解释: M = 1000, CM = 900, XC = 90, IV = 4.
    /// + 提示：
    ///     - 1 <= s.length <= 15
    ///     - s 仅含字符 ('I', 'V', 'X', 'L', 'C', 'D', 'M')
    ///     - 题目数据保证 s 是一个有效的罗马数字，且表示整数在范围 `[1, 3999]` 内
    ///     - 题目所给测试用例皆符合罗马数字书写规则，不会出现跨位等情况。
    ///     - IL 和 IM 这样的例子并不符合题目要求，49 应该写作 XLIX，999 应该写作 CMXCIX 。
    ///     - 关于罗马数字的详尽书写规则，可以参考 [罗马数字 - Mathematics](https://b2b.partcommunity.com/community/knowledge/zh_CN/detail/10753/%E7%BD%97%E9%A9%AC%E6%95%B0%E5%AD%97#knowledge_article) 。
    ///
    /// Related Topics 数学 字符串
    ///
    /// 👍 1320 👎 0
    ///
    /// # 方法1 模式匹配
    /// ```rust
    /// pub fn roman_to_int(s: String) -> i32 {
    ///     if s.is_empty() {
    ///         return 0;
    ///     }
    ///     let mut sum = 0;
    ///     let s = s.into_bytes();
    ///     let mut pre = 0;
    ///     for i in 0..s.len() {
    ///         let num = match s[i] {
    ///             b'I' => 1,
    ///             b'V' => 5,
    ///             b'X' => 10,
    ///             b'L' => 50,
    ///             b'C' => 100,
    ///             b'D' => 500,
    ///             b'M' => 1000,
    ///             _ => 0
    ///         };
    ///         if pre < num && pre != 0 {
    ///             sum -= pre;
    ///         } else {
    ///             sum += pre;
    ///         }
    ///         pre = num;
    ///     }
    ///     sum += pre;
    ///     sum
    /// }
    /// ```
    /// # 方法2 查表法
    pub fn roman_to_int(s: String) -> i32 {
        const INT_TO_ROMAN: [(i32, &[u8]); 13] = [
            (1000, b"M"),
            (900, b"CM"),
            (500, b"D"),
            (400, b"CD"),
            (100, b"C"),
            (90, b"XC"),
            (50, b"L"),
            (40, b"XL"),
            (10, b"X"),
            (9, b"IX"),
            (5, b"V"),
            (4, b"IV"),
            (1, b"I"),
        ];
        let s = s.into_bytes();
        let len = s.len();
        let mut i = 0;
        let mut ret = 0;
        for &(int, roman) in &INT_TO_ROMAN {
            let roman_len = roman.len();
            while s[i..].starts_with(roman) {
                ret += int;
                i += roman_len;
                if i >= len {
                    return ret;
                }
            }
        }
        0
    }
}


#[cfg(test)]
mod test {
    use crate::Solution;

    #[test]
    fn test_q0013() {
        let s = "MCMXCIV".to_string();
        let n = Solution::roman_to_int(s);
        assert_eq!(n, 1994);
    }
}
