use crate::Solution;

impl Solution {
    /// [14.最长公共前缀](https://leetcode-cn.com/problems/longest-common-prefix/)
    ///
    /// 2022-02-18 14:23:27
    ///
    /// 编写一个函数来查找字符串数组中的最长公共前缀。
    ///
    /// 如果不存在公共前缀，返回空字符串 `""`。
    ///
    /// + **示例 1：**
    ///     + **输入：** strs = \["flower","flow","flight"\]
    ///     + **输出：** "fl"
    /// + **示例 2：**
    ///     + **输入：** strs = \["dog","racecar","car"\]
    ///     + **输出：** ""
    ///     + **解释：** 输入不存在公共前缀。
    /// + **提示：**
    ///     * `1 <= strs.length <= 200`
    ///     * `0 <= strs[i].length <= 200`
    ///     * `strs[i]` 仅由小写英文字母组成
    /// + Related Topics
    ///     * 字符串
    /// * 👍 2033
    /// * 👎 0
    ///
    /// 1. 排序
    ///
    ///     排序后，返回第一个和最后一个的公共前缀
    ///
    ///     ```rust
    ///     pub fn longest_common_prefix(mut strs: Vec<String>) -> String {
    ///         if strs.is_empty() {
    ///             return "".to_string();
    ///         }
    ///         strs.sort();
    ///         let first = strs.first().unwrap().as_bytes();
    ///         let last = strs.last().unwrap().as_bytes();
    ///         let n = std::cmp::min(first.len(), last.len());
    ///         let mut ret = String::with_capacity(n);
    ///         for i in 0..n {
    ///             if first[i] != last[i] {
    ///                 break;
    ///             }
    ///             ret.push(first[i] as char);
    ///         }
    ///         ret
    ///     }
    ///     ```
    /// 2. 横向扫描
    ///
    ///     每两个之间的公共前缀。
    ///
    ///     ```rust
    ///     pub fn longest_common_prefix(mut strs: Vec<String>) -> String {
    ///         fn lcp(str1: &String, str2: &String) -> String {
    ///             let n = std::cmp::min(str1.len(), str2.len());
    ///             let b1 = str1.as_bytes();
    ///             let b2 = str2.as_bytes();
    ///             let mut ret = String::new();
    ///             for i in 0..n {
    ///                 if b1[i] != b2[i] {
    ///                     break;
    ///                 }
    ///                 ret.push(b1[i] as char);
    ///             }
    ///             ret
    ///         }
    ///         if strs.is_empty() {
    ///             return "".to_string();
    ///         }
    ///         let mut prefix = strs[0].clone();
    ///         for i in 1..strs.len() {
    ///             prefix = lcp(&prefix, &strs[i]);
    ///             if prefix.is_empty() {
    ///                 break;
    ///             }
    ///         }
    ///         prefix
    ///     }
    ///     ```
    /// 3. 纵向扫描
    ///
    ///     从0开始每个字符是否相等。
    ///
    ///     ```rust
    ///     pub fn longest_common_prefix(strs: Vec<String>) -> String {
    ///         if strs.is_empty() {
    ///             return "".to_string();
    ///         }
    ///         let length = strs[0].len();
    ///         let count = strs.len();
    ///         for i in 0..length {
    ///             let b = strs[0].as_bytes()[i];
    ///             for j in 1..count {
    ///                 if i == strs[j].len() || strs[j].as_bytes()[i] != b {
    ///                     return strs[0].split_at(i).0.clone().to_string();
    ///                 }
    ///             }
    ///         }
    ///         return strs[0].clone();
    ///     }
    ///     ```
    /// 4. 分治
    ///
    ///     两两之间的公共前缀。
    ///
    ///     ```rust
    ///     pub fn longest_common_prefix(strs: Vec<String>) -> String {
    ///         fn lcp(str1: &String, str2: &String) -> String {
    ///             let n = std::cmp::min(str1.len(), str2.len());
    ///             let b1 = str1.as_bytes();
    ///             let b2 = str2.as_bytes();
    ///             let mut ret = String::new();
    ///             for i in 0..n {
    ///                 if b1[i] != b2[i] {
    ///                     break;
    ///                 }
    ///                 ret.push(b1[i] as char);
    ///             }
    ///             ret
    ///         }
    ///         fn lcp_r(strs: &Vec<String>, start: usize, end: usize) -> String {
    ///             if start == end {
    ///                 strs[start].clone()
    ///             } else {
    ///                 let mid = (end - start) / 2 + start;
    ///                 let lcp_left = lcp_r(strs, start, mid);
    ///                 let lcp_right = lcp_r(strs, mid + 1, end);
    ///                 lcp(&lcp_left, &lcp_right)
    ///             }
    ///         }
    ///         if strs.is_empty() {
    ///             "".to_string()
    ///         } else {
    ///             lcp_r(&strs, 0, strs.len() - 1).to_owned()
    ///         }
    ///     }
    ///     ```
    /// 5. 二分查找
    /// 6. 字典树
    pub fn longest_common_prefix(strs: Vec<String>) -> String {
        if strs.is_empty() {
            return "".to_string();
        }
        let length = strs[0].len();
        let count = strs.len();
        for i in 0..length {
            let b = strs[0].as_bytes()[i];
            for j in 1..count {
                if i == strs[j].len() || strs[j].as_bytes()[i] != b {
                    return strs[0].split_at(i).0.clone().to_string();
                }
            }
        }
        return strs[0].clone();
    }
}


#[cfg(test)]
mod test {
    use crate::Solution;

    #[test]
    fn test_q0014() {
        let strs = vec!["flower".to_string(), "flow".to_string(), "flight".to_string()];
        assert_eq!(Solution::longest_common_prefix(strs), "fl".to_string());
    }
}
