use crate::Solution;

impl Solution {
    /// [399.除法求值](https://leetcode-cn.com/problems/evaluate-division/)
    ///
    /// 2022-02-21 10:35:22
    ///
    /// 给你一个变量对数组 `equations` 和一个实数值数组 `values` 作为已知条件，其中 `equations[i] = [Ai, Bi]` 和 `values[i]` 共同表示等式 `Ai / Bi = values[i]` 。每个 `Ai` 或 `Bi` 是一个表示单个变量的字符串。
    ///
    /// 另有一些以数组 `queries` 表示的问题，其中 `queries[j] = [Cj, Dj]` 表示第 `j` 个问题，请你根据已知条件找出 `Cj / Dj = ?` 的结果作为答案。
    ///
    /// 返回 **所有问题的答案** 。如果存在某个无法确定的答案，则用 `-1.0` 替代这个答案。如果问题中出现了给定的已知条件中没有出现的字符串，也需要用 `-1.0` 替代这个答案。
    ///
    /// **注意：** 输入总是有效的。你可以假设除法运算中不会出现除数为 0 的情况，且不存在任何矛盾的结果。
    ///
    /// + **示例 1：**
    ///     + **输入：** equations = \[\["a","b"\],\["b","c"\]\], values = \[2.0,3.0\], queries = \[\["a","c"\],\["b","a"\],\["a","e"\],\["a","a"\],\["x","x"\]\]
    ///     + **输出：** \[6.00000,0.50000,-1.00000,1.00000,-1.00000\]
    ///     + **解释：**
    ///         - 条件：_a / b = 2.0_, _b / c = 3.0_
    ///         - 问题：_a / c = ?_, _b / a = ?_, _a / e = ?_, _a / a = ?_, _x / x = ?_
    ///         - 结果：\[6.0, 0.5, -1.0, 1.0, -1.0 \]
    /// + **示例 2：**
    ///     + **输入：** equations = \[\["a","b"\],\["b","c"\],\["bc","cd"\]\], values = \[1.5,2.5,5.0\], queries = \[\["a","c"\],\["c","b"\],\["bc","cd"\],\["cd","bc"\]\]
    ///     + **输出：** \[3.75000,0.40000,5.00000,0.20000\]
    /// + **示例 3：**
    ///     + **输入：** equations = \[\["a","b"\]\], values = \[0.5\], queries = \[\["a","b"\],\["b","a"\],\["a","c"\],\["x","y"\]\]
    ///     + **输出：** \[0.50000,2.00000,-1.00000,-1.00000\]
    /// + **提示：**
    ///     * `1 <= equations.length <= 20`
    ///     * `equations[i].length == 2`
    ///     * `1 <= Ai.length, Bi.length <= 5`
    ///     * `values.length == equations.length`
    ///     * `0.0 < values[i] <= 20.0`
    ///     * `1 <= queries.length <= 20`
    ///     * `queries[i].length == 2`
    ///     * `1 <= Cj.length, Dj.length <= 5`
    ///     * `Ai, Bi, Cj, Dj` 由小写英文字母与数字组成
    /// + Related Topics
    ///     * 深度优先搜索
    ///     * 广度优先搜索
    ///     * 并查集
    ///     * 图
    ///     * 数组
    ///     * 最短路
    /// * 👍 681
    /// * 👎 0
    ///
    /// 1. 广度优先搜索
    ///     转为加权图，从起点开始通过广度优先搜索，不断更新起点与当前点之间的路径长度，直到搜索到终点为止。实现见源码。
    ///     1. 复杂度分析
    ///         - 时间复杂度：O(ML+Q(L+M)，M为边的数量，Q为询问的数量，L为字符串的平均长度。构建图时，需要处理M条边，每
    ///         条边都涉及到O(L)的字符串比较；处理查询时，每次查询首先要进行一次O(L)的比较，然后至多遍历O(M)条边。
    ///         - 空间复杂度：O(NL+M)，其中N为点的数量，M为边的数量，L为字符串的平均长度。为了将每个字符串映射到整数，需
    ///         要开辟空间为O(NL)的哈希表；随后，需要花费O(M)的空间存储每条边的权重；处理查询时，还需要O(N)的空间维护访
    ///         问队列。最终，总的复杂度为O(NL+M+N)=O(NL+M)。
    ///     2. 源码
    ///         ```rust
    ///         pub fn calc_equation(equations: Vec<Vec<String>>, values: Vec<f64>, queries: Vec<Vec<String>>) -> Vec<f64> {
    ///             let mut nvars = 0usize;
    ///             let mut variables = std::collections::HashMap::<String, usize>::new();
    ///             let n = equations.len();
    ///             for i in 0..n {
    ///                 if !variables.contains_key(&*equations[i][0]) {
    ///                     variables.insert(equations[i][0].clone(), nvars);
    ///                     nvars += 1usize;
    ///                 }
    ///                 if !variables.contains_key(&*equations[i][1]) {
    ///                     variables.insert(equations[i][1].clone(), nvars);
    ///                     nvars += 1usize;
    ///                 }
    ///             }
    ///             let mut edges = vec![Vec::<(usize, f64)>::new(); nvars];
    ///             for i in 0..n {
    ///                 let (va, vb) = (variables[&*equations[i][0]], variables[&*equations[i][1]]);
    ///                 edges[va].push((vb, values[i]));
    ///                 edges[vb].push((va, 1.0f64 / values[i]));
    ///             }
    ///             let mut ret = Vec::<f64>::new();
    ///             for q in queries {
    ///                 let mut result = -1.0;
    ///                 if variables.contains_key(&*q[0]) && variables.contains_key(&*q[1]) {
    ///                     let (ia, ib) = (variables[&*q[0]], variables[&*q[1]]);
    ///                     if ia == ib {
    ///                         result = 1.0;
    ///                     } else {
    ///                         let mut points = Vec::new();
    ///                         points.push(ia);
    ///                         let mut ratios = vec![-1.0; nvars];
    ///                         ratios[ia] = 1.0;
    ///                         while !points.is_empty() && ratios[ib] < 0f64 {
    ///                             let x = points.remove(0);
    ///                             for (y, val) in &edges[x] {
    ///                                 if ratios[*y] < 0f64 {
    ///                                     ratios[*y] = ratios[x] * *val;
    ///                                     points.push(*y);
    ///                                 }
    ///                             }
    ///                         }
    ///                         result = ratios[ib];
    ///                     }
    ///                 }
    ///                 ret.push(result)
    ///             }
    ///             ret
    ///         }
    ///         ```
    /// 2. 弗洛伊德算法
    ///     ```rust
    ///     pub fn calc_equation(equations: Vec<Vec<String>>, values: Vec<f64>, queries: Vec<Vec<String>>) -> Vec<f64> {
    ///         let mut nvars = 0usize;
    ///         let mut variables = std::collections::HashMap::<String, usize>::new();
    ///         let n = equations.len();
    ///         for i in 0..n {
    ///             if !variables.contains_key(&*equations[i][0]) {
    ///                 variables.insert(equations[i][0].clone(), nvars);
    ///                 nvars += 1usize;
    ///             }
    ///             if !variables.contains_key(&*equations[i][1]) {
    ///                 variables.insert(equations[i][1].clone(), nvars);
    ///                 nvars += 1usize;
    ///             }
    ///         }
    ///         let mut graph = vec![vec![-1.0; nvars]; nvars];
    ///         for i in 0..n{
    ///             let (va, vb) = (variables[&*equations[i][0]], variables[&*equations[i][1]]);
    ///             graph[va][vb] = values[i];
    ///             graph[vb][va] = 1.0f64 / values[i];
    ///         }
    ///         for k in 0..nvars {
    ///             for i in 0..nvars {
    ///                 for j in 0..nvars {
    ///                     if graph[i][k] > 0.0 && graph[k][j] > 0.0 {
    ///                         graph[i][j] = graph[i][k] * graph[k][j];
    ///                     }
    ///                 }
    ///             }
    ///         }
    ///         let mut ret = Vec::<f64>::new();
    ///         for q in queries {
    ///             let mut result = -1.0;
    ///             if variables.contains_key(&*q[0]) && variables.contains_key(&*q[1]) {
    ///                 let (ia, ib) = (variables[&*q[0]], variables[&*q[1]]);
    ///                 if graph[ia][ib] > 0.0 {
    ///                     result = graph[ia][ib];
    ///                 }
    ///             }
    ///             ret.push(result);
    ///         }
    ///         ret
    ///     }
    ///     ```
    /// # 方法3 加权并查集
    ///     源码见实现
    pub fn calc_equation(equations: Vec<Vec<String>>, values: Vec<f64>, queries: Vec<Vec<String>>) -> Vec<f64> {
        fn find(f: &mut Vec<usize>, w: &mut Vec<f64>, x: usize) -> usize {
            if f[x] != x {
                let father = find(f, w, f[x]);
                w[x] = w[x] * w[f[x]];
                f[x] = father;
            }
            f[x]
        }
        fn merge(f: &mut Vec<usize>, w: &mut Vec<f64>, x: usize, y: usize, val: f64) {
            let fx = find(f, w, x);
            let fy = find(f, w, y);
            f[fx] = fy;
            w[fx] = val * w[y] / w[x];
        }

        let mut nvars = 0usize;
        let mut variables = std::collections::HashMap::<String, usize>::new();
        let n = equations.len();
        for i in 0..n {
            if !variables.contains_key(&*equations[i][0]) {
                variables.insert(equations[i][0].clone(), nvars);
                nvars += 1usize;
            }
            if !variables.contains_key(&*equations[i][1]) {
                variables.insert(equations[i][1].clone(), nvars);
                nvars += 1usize;
            }
        }
        let mut f = (0..nvars).collect::<Vec<usize>>();
        let mut w = vec![1.0; nvars];

        for i in 0..n {
            let (va, vb) = (variables[&*equations[i][0]], variables[&*equations[i][1]]);
            merge(&mut f, &mut w, va, vb, values[i]);
        }

        let mut ret = Vec::<f64>::new();

        for q in queries {
            let mut result = -1.0;
            if variables.contains_key(&*q[0]) && variables.contains_key(&*q[1]) {
                let (ia, ib) = (variables[&*q[0]], variables[&*q[1]]);
                let (fa, fb) = (find(&mut f, &mut w, ia), find(&mut f, &mut w, ib));
                if fa == fb {
                    result = w[ia] / w[ib];
                }
            }
            ret.push(result);
        }

        ret
    }
}

#[cfg(test)]
mod test {
    use crate::Solution;

    #[test]
    fn test_q0399() {
        let eq = vec![vec!["a".to_string(), "b".to_string()], vec!["b".to_string(), "c".to_string()]];
        let val = vec![2.0, 3.0];
        let que = vec![
            vec!["a".to_string(), "c".to_string()],
            vec!["b".to_string(), "a".to_string()],
            vec!["a".to_string(), "e".to_string()],
            vec!["a".to_string(), "a".to_string()],
            vec!["x".to_string(), "x".to_string()]
        ];

        let vec1 = Solution::calc_equation(eq, val, que);

        //println!("{:?}", vec1);
        assert_eq!(vec1, vec![6.0, 0.5, -1.0, 1.0, -1.0])
    }
}
