use crate::Solution;

#[derive(Default)]
struct Trie(Option<Box<Trie>>, Option<Box<Trie>>);

impl Trie {
    fn new() -> Self {
        Default::default()
    }

    fn insert(&mut self, num: i32) {
        (0..31).rev().fold(self, |node, k|
            match (num >> k) & 1 {
                0 => node.0.get_or_insert_with(Default::default),
                _ => node.1.get_or_insert_with(Default::default)
            });
    }

    fn check(&self, num: i32) -> i32 {
        let mut c = self;
        let mut x = 0;
        for k in (0..31).rev() {
            if ((num >> k) & 1) == 0 {
                if let Some(r) = &c.1 {
                    c = r;
                    x = (x << 1) + 1;
                } else if let Some(l) = &c.0 {
                    c = l;
                    x <<= 1;
                }
            } else {
                if let Some(l) = &c.0 {
                    c = l;
                    x = (x << 1) + 1;
                } else if let Some(r) = &c.1 {
                    c = r;
                    x <<= 1;
                }
            }
        }
        x
    }
}

impl Solution {
    /// [421.数组中两个数的最大异或值](https://leetcode-cn.com/problems/maximum-xor-of-two-numbers-in-an-array/)
    ///
    /// 2021-05-16 22:13:21
    ///
    /// 给你一个整数数组 `nums` ，返回 `nums[i] XOR nums[j]` 的最大运算结果，其中 `0 ≤ i ≤ j < n` 。
    ///
    /// 进阶：你可以在 `O(n)` 的时间解决这个问题吗？
    ///
    /// + 示例 1：
    ///     - 输入：`nums = [3,10,5,25,2,8]`
    ///     - 输出：`28`
    ///     - 解释：最大运算结果是 `5 XOR 25 = 28`.
    /// + 示例 2：
    ///     - 输入：`nums = [0]`
    ///     - 输出：`0`
    /// + 示例 3：
    ///     - 输入：`nums = [2,4]`
    ///     - 输出：`6`
    /// + 示例 4：
    ///     - 输入：`nums = [8,10,2]`
    ///     - 输出：`10`
    /// + 示例 5：
    ///     - 输入：`nums = [14,70,53,83,49,91,36,80,92,51,66,70]`
    ///     - 输出：`127`
    /// + 提示：
    ///     - `1 <= nums.length <= 2 * 104`
    ///     - `0 <= nums[i] <= 231 - 1`
    ///
    /// Related Topics 位运算 字典树
    ///
    /// 👍 334 👎 0
    ///
    /// # 方法1
    /// 暴力模拟
    /// ```
    /// pub fn find_maximum_xor(nums: Vec<i32>) -> i32 {
    ///     let mut ans = 0;
    ///     for i in 0..nums.len() {
    ///         for j in i..nums.len() {
    ///             ans = ans.max(nums[i] ^ nums[j]);
    ///         }
    ///     }
    ///     ans
    /// }
    /// ```
    /// # 方法2
    /// 字典树
    pub fn find_maximum_xor(nums: Vec<i32>) -> i32 {
        let mut x = 0;
        let mut trie = Trie::new();
        for i in 1..nums.len() {
            trie.insert(nums[i - 1]);
            x = x.max(trie.check(nums[i]));
        }
        x
    }
}


#[cfg(test)]
mod test {
    use crate::Solution;

    #[test]
    fn test_q0421() {
        let nums = vec![14, 70, 53, 83, 49, 91, 36, 80, 92, 51, 66, 70];
        let ans = Solution::find_maximum_xor(nums);
        assert_eq!(ans, 127);
    }
}
