use crate::Solution;

impl Solution {
    /// [547.省份数量](https://leetcode-cn.com/problems/number-of-provinces/)
    ///
    /// 2022-02-21 11:09:51
    ///
    /// 有 `n` 个城市，其中一些彼此相连，另一些没有相连。如果城市 `a` 与城市 `b` 直接相连，且城市 `b` 与城市 `c` 直接相连，那么城市 `a` 与城市 `c` 间接相连。
    ///
    /// **省份** 是一组直接或间接相连的城市，组内不含其他没有相连的城市。
    ///
    /// 给你一个 `n x n` 的矩阵 `isConnected` ，其中 `isConnected[i][j] = 1` 表示第 `i` 个城市和第 `j` 个城市直接相连，而 `isConnected[i][j] = 0` 表示二者不直接相连。
    ///
    /// 返回矩阵中 **省份** 的数量。
    ///
    /// + **示例 1：**
    ///     + ![](https://assets.leetcode.com/uploads/2020/12/24/graph1.jpg)
    ///     + **输入：** isConnected = \[\[1,1,0\],\[1,1,0\],\[0,0,1\]\]
    ///     + **输出：** 2
    /// + **示例 2：**
    ///     + ![](https://assets.leetcode.com/uploads/2020/12/24/graph2.jpg)
    ///     + **输入：** isConnected = \[\[1,0,0\],\[0,1,0\],\[0,0,1\]\]
    ///     + **输出：** 3
    /// + **提示：**
    ///     * `1 <= n <= 200`
    ///     * `n == isConnected.length`
    ///     * `n == isConnected[i].length`
    ///     * `isConnected[i][j]` 为 `1` 或 `0`
    ///     * `isConnected[i][i] == 1`
    ///     * `isConnected[i][j] == isConnected[j][i]`
    /// + Related Topics
    ///     * 深度优先搜索
    ///     * 广度优先搜索
    ///     * 并查集
    ///     * 图
    /// * 👍 716
    /// * 👎 0
    ///
    /// 1. 深度优先搜索
    ///     ```rust
    ///     pub fn find_circle_num(is_connected: Vec<Vec<i32>>) -> i32 {
    ///         let provinces = is_connected.len();
    ///         let mut visited = vec![false; provinces];
    ///         let mut circles = 0;
    ///         fn dfs(is_connected: &Vec<Vec<i32>>, visited:&mut Vec<bool>, provinces: usize, i: usize) {
    ///             for j in 0..provinces {
    ///                 if is_connected[i][j] == 1 && !visited[j] {
    ///                     visited[j] = true;
    ///                     dfs(is_connected, visited, provinces, j);
    ///                 }
    ///             }
    ///         }
    ///         for i in 0..provinces {
    ///             if !visited[i] {
    ///                 dfs(&is_connected, &mut visited, provinces, i);
    ///                 circles += 1;
    ///             }
    ///         }
    ///         circles
    ///     }
    ///     ```
    /// 2. 广度优先搜索
    ///     ```rust
    ///     pub fn find_circle_num(is_connected: Vec<Vec<i32>>) -> i32 {
    ///         let provinces = is_connected.len();
    ///         let mut visited = vec![false; provinces];
    ///         let mut circles = 0;
    ///         let mut queue = Vec::<usize>::new();
    ///         for i in 0..provinces {
    ///             if !visited[i] {
    ///                 queue.push(i);
    ///                 while !queue.is_empty() {
    ///                     let j = queue.remove(0);
    ///                     visited[j] = true;
    ///                     for k in 0..provinces {
    ///                         if is_connected[j][k] == 1 && !visited[k] {
    ///                             queue.push(k);
    ///                         }
    ///                     }
    ///                 }
    ///                 circles += 1;
    ///             }
    ///         }
    ///         circles
    ///     }
    ///     ```
    /// 3. 并查集
    ///
    ///     实现方法见源码
    pub fn find_circle_num(is_connected: Vec<Vec<i32>>) -> i32 {
        let provinces = is_connected.len();
        let mut parent = (0..provinces).collect::<Vec<usize>>();
        fn find(parent: &mut Vec<usize>, idx: usize) -> usize {
            if parent[idx] != idx {
                parent[idx] = find(parent, parent[idx]);
            }
            parent[idx]
        }
        fn union(parent: &mut Vec<usize>, idx1: usize, idx2: usize) {
            let a = find(parent, idx1);
            let b = find(parent, idx2);
            parent[a] = b;
        }

        for i in 0..provinces {
            for j in i + 1..provinces {
                if is_connected[i][j] == 1 {
                    union(&mut parent, i, j);
                }
            }
        }

        let mut circles = 0;

        for i in 0..provinces {
            if parent[i] == i {
                circles += 1;
            }
        }

        circles
    }
}

#[cfg(test)]
mod test {
    use crate::Solution;

    #[test]
    fn test_q0547() {
        let c = vec![vec![1, 1, 0], vec![1, 1, 0], vec![0, 0, 1]];
        let n = Solution::find_circle_num(c);
        assert_eq!(n, 2);
    }
}
