use crate::Solution;

impl Solution {
    /// [684.冗余连接](https://leetcode-cn.com/problems/redundant-connection/)
    ///
    /// 2022-02-21 16:49:22
    ///
    /// 树可以看成是一个连通且 **无环** 的 **无向** 图。
    ///
    /// 给定往一棵 `n` 个节点 (节点值 `1～n`) 的树中添加一条边后的图。添加的边的两个顶点包含在 `1` 到 `n` 中间，且这条附加的边不属于树中已存在的边。图的信息记录于长度为 `n` 的二维数组 `edges` ，`edges[i] = [ai, bi]` 表示图中在 `ai` 和 `bi` 之间存在一条边。
    ///
    /// 请找出一条可以删去的边，删除后可使得剩余部分是一个有着 `n` 个节点的树。如果有多个答案，则返回数组 `edges` 中最后出现的边。
    ///
    /// + **示例 1：**
    ///     + ![](https://pic.leetcode-cn.com/1626676174-hOEVUL-image.png)
    ///     + **输入:** edges = \[\[1,2\], \[1,3\], \[2,3\]\]
    ///     + **输出:** \[2,3\]
    /// + **示例 2：**
    ///     + ![](https://pic.leetcode-cn.com/1626676179-kGxcmu-image.png)
    ///     + **输入:** edges = \[\[1,2\], \[2,3\], \[3,4\], \[1,4\], \[1,5\]\]
    ///     + **输出:** \[1,4\]
    /// + **提示:**
    ///     * `n == edges.length`
    ///     * `3 <= n <= 1000`
    ///     * `edges[i].length == 2`
    ///     * `1 <= ai < bi <= edges.length`
    ///     * `ai != bi`
    ///     * `edges` 中无重复元素
    ///     * 给定的图是连通的
    /// + Related Topics
    ///     * 深度优先搜索
    ///     * 广度优先搜索
    ///     * 并查集
    ///     * 图
    /// * 👍 431
    /// * 👎 0
    pub fn find_redundant_connection(edges: Vec<Vec<i32>>) -> Vec<i32> {
        fn find(parent: &mut Vec<usize>, index: usize) -> usize {
            if parent[index] != index {
                parent[index] = find(parent, parent[index]);
            }
            parent[index]
        }
        fn union(parent: &mut Vec<usize>, index1: usize, index2: usize) {
            let idx1 = find(parent, index1);
            let idx2 = find(parent, index2);
            parent[idx1] = idx2;
        }

        let nodes_count = edges.len();
        let mut parent = (0..nodes_count + 1).collect::<Vec<usize>>();
        for edge in edges {
            let (node1, node2) = (edge[0] as usize, edge[1] as usize);
            if find(&mut parent, node1) != find(&mut parent, node2) {
                union(&mut parent, node1, node2);
            } else {
                return edge;
            }
        }
        Vec::<i32>::new()
    }
}

#[cfg(test)]
mod test {
    use crate::Solution;

    #[test]
    fn test_q0684() {
        let edges = vec![vec![1, 2], vec![2, 3], vec![3, 4], vec![1, 4], vec![1, 5]];
        let redundant_connection = Solution::find_redundant_connection(edges);
        //println!("{:?}", redundant_connection);
        assert_eq!(redundant_connection, vec![1, 4]);
    }
}
