use crate::Solution;

impl Solution {
    /// [721.账户合并](https://leetcode-cn.com/problems/accounts-merge/)
    ///
    /// 2022-02-21 16:56:57
    ///
    /// 给定一个列表 `accounts`，每个元素 `accounts[i]` 是一个字符串列表，其中第一个元素 `accounts[i][0]` 是 _名称 (name)_，其余元素是 _**emails**_ 表示该账户的邮箱地址。
    ///
    /// 现在，我们想合并这些账户。如果两个账户都有一些共同的邮箱地址，则两个账户必定属于同一个人。请注意，即使两个账户具有相同的名称，它们也可能属于不同的人，因为人们可能具有相同的名称。一个人最初可以拥有任意数量的账户，但其所有账户都具有相同的名称。
    ///
    /// 合并账户后，按以下格式返回账户：每个账户的第一个元素是名称，其余元素是 **按字符 ASCII 顺序排列** 的邮箱地址。账户本身可以以 **任意顺序** 返回。
    ///
    /// + **示例 1：**
    ///     + **输入：** accounts = \[\["John", "johnsmith@mail.com", "john00@mail.com"\], \["John", "johnnybravo@mail.com"\], \["John", "johnsmith@mail.com", "john\_newyork@mail.com"\], \["Mary", "mary@mail.com"\]\]
    ///     + **输出：** \[\["John", 'john00@mail.com', 'john\_newyork@mail.com', 'johnsmith@mail.com'\],  \["John", "johnnybravo@mail.com"\], \["Mary", "mary@mail.com"\]\]
    ///     + **解释：**
    ///         - 第一个和第三个 John 是同一个人，因为他们有共同的邮箱地址 "johnsmith@mail.com"。
    ///         - 第二个 John 和 Mary 是不同的人，因为他们的邮箱地址没有被其他帐户使用。
    ///         - 可以以任何顺序返回这些列表，例如答案 \[\['Mary'，'mary@mail.com'\]，\['John'，'johnnybravo@mail.com'\]，
    ///         - \['John'，'john00@mail.com'，'john\_newyork@mail.com'，'johnsmith@mail.com'\]\] 也是正确的。
    /// + **示例 2：**
    ///     + **输入：** accounts = \[\["Gabe","Gabe0@m.co","Gabe3@m.co","Gabe1@m.co"\],\["Kevin","Kevin3@m.co","Kevin5@m.co","Kevin0@m.co"\],\["Ethan","Ethan5@m.co","Ethan4@m.co","Ethan0@m.co"\],\["Hanzo","Hanzo3@m.co","Hanzo1@m.co","Hanzo0@m.co"\],\["Fern","Fern5@m.co","Fern1@m.co","Fern0@m.co"\]\]
    ///     + **输出：** \[\["Ethan","Ethan0@m.co","Ethan4@m.co","Ethan5@m.co"\],\["Gabe","Gabe0@m.co","Gabe1@m.co","Gabe3@m.co"\],\["Hanzo","Hanzo0@m.co","Hanzo1@m.co","Hanzo3@m.co"\],\["Kevin","Kevin0@m.co","Kevin3@m.co","Kevin5@m.co"\],\["Fern","Fern0@m.co","Fern1@m.co","Fern5@m.co"\]\]
    /// + **提示：**
    ///     * `1 <= accounts.length <= 1000`
    ///     * `2 <= accounts[i].length <= 10`
    ///     * `1 <= accounts[i][j].length <= 30`
    ///     * `accounts[i][0]` 由英文字母组成
    ///     * `accounts[i][j] (for j > 0)` 是有效的邮箱地址
    /// + Related Topics
    ///     * 深度优先搜索
    ///     * 广度优先搜索
    ///     * 并查集
    ///     * 数组
    ///     * 字符串
    /// * 👍 344
    /// * 👎 0
    pub fn accounts_merge(accounts: Vec<Vec<String>>) -> Vec<Vec<String>> {
        fn find(parent: &mut Vec<usize>, index: usize) -> usize {
            if parent[index] != index {
                parent[index] = find(parent, parent[index]);
            }
            parent[index]
        }
        fn union(parent: &mut Vec<usize>, index1: usize, index2: usize) {
            let idx1 = find(parent, index1);
            let idx2 = find(parent, index2);
            parent[idx2] = idx1;
        }
        use std::collections::HashMap;
        let mut email_to_idx = HashMap::<String, usize>::new();
        let mut email_to_name = HashMap::<String, String>::new();
        let mut emails_count = 0usize;
        for account in &accounts {
            let name = &account[0];
            for i in 1..account.len() {
                let email = &account[i];
                if !email_to_idx.contains_key(email) {
                    email_to_idx.insert(email.clone(), emails_count);
                    email_to_name.insert(email.clone(), name.clone());
                    emails_count += 1;
                }
            }
        }
        let mut parent = (0..emails_count).collect::<Vec<usize>>();
        for account in &accounts {
            let first_email = &account[1];
            let first_idx = email_to_idx[first_email];
            for i in 2..account.len() {
                let next_email = &account[i];
                let next_idx = email_to_idx[next_email];
                union(&mut parent, first_idx, next_idx);
            }
        }
        let mut idx_to_emails = HashMap::<usize, Vec<String>>::new();
        for email in email_to_idx.keys() {
            let idx = find(&mut parent, email_to_idx[email]);
            if !idx_to_emails.contains_key(&idx) {
                idx_to_emails.insert(idx, Vec::<String>::new());
            }
            idx_to_emails.get_mut(&idx).unwrap().push(email.clone());
        }
        let mut merged = Vec::<Vec<String>>::new();
        for emails in idx_to_emails.values_mut() {
            emails.sort();
            let name = &email_to_name[&emails[0]];
            let mut account = Vec::<String>::with_capacity(1usize + emails.len());
            account.push(name.clone());
            account.extend(emails.clone());
            merged.push(account);
        }
        merged
    }
}

#[cfg(test)]
mod test {
    use crate::Solution;

    #[test]
    fn test_q0721() {
        let accounts = vec![
            vec!["John".to_string(), "johnsmith@mail.com".to_string(), "john00@mail.com".to_string()],
            vec!["John".to_string(), "johnnybravo@mail.com".to_string()],
            vec!["John".to_string(), "johnsmith@mail.com".to_string(), "john_newyork@mail.com".to_string()],
            vec!["Mary".to_string(), "mary@mail.com".to_string()]];
        let ans = vec![
            vec!["Mary".to_string(), "mary@mail.com".to_string()],
            vec!["John".to_string(), "john00@mail.com".to_string(), "john_newyork@mail.com".to_string(), "johnsmith@mail.com".to_string()],
            vec!["John".to_string(), "johnnybravo@mail.com".to_string()]];
        let merged = Solution::accounts_merge(accounts);
        for a in &merged {
            if !ans.contains(a) {
                panic!()
            }
        }
    }
}
