use crate::Solution;

impl Solution {
    /// [778.水位上升的泳池中游泳](https://leetcode-cn.com/problems/swim-in-rising-water/)
    ///
    /// 2022-02-21 17:14:26
    ///
    /// 在一个 `n x n` 的整数矩阵 `grid` 中，每一个方格的值 `grid[i][j]` 表示位置 `(i, j)` 的平台高度。
    ///
    /// 当开始下雨时，在时间为 `t` 时，水池中的水位为 `t` 。你可以从一个平台游向四周相邻的任意一个平台，但是前提是此时水位必须同时淹没这两个平台。假定你可以瞬间移动无限距离，也就是默认在方格内部游动是不耗时的。当然，在你游泳的时候你必须待在坐标方格里面。
    ///
    /// 你从坐标方格的左上平台 `(0，0)` 出发。返回 _你到达坐标方格的右下平台 `(n-1, n-1)` 所需的最少时间 。_
    ///
    /// + **示例 1:**
    ///     - ![](https://assets.leetcode.com/uploads/2021/06/29/swim1-grid.jpg)
    ///     - **输入:** grid = \[\[0,2\],\[1,3\]\]
    ///     - **输出:** 3
    ///     - **解释:**
    ///         - 时间为0时，你位于坐标方格的位置为 `(0, 0)。`
    ///         - 此时你不能游向任意方向，因为四个相邻方向平台的高度都大于当前时间为 0 时的水位。
    ///         - 等时间到达 3 时，你才可以游向平台 (1, 1). 因为此时的水位是 3，坐标方格中的平台没有比水位 3 更高的，所以你可以游向坐标方格中的任意位置
    /// + **示例 2:**
    ///     - ![](https://assets.leetcode.com/uploads/2021/06/29/swim2-grid-1.jpg)
    ///     - **输入:** grid = \[\[0,1,2,3,4\],\[24,23,22,21,5\],\[12,13,14,15,16\],\[11,17,18,19,20\],\[10,9,8,7,6\]\]
    ///     - **输出:** 16
    ///     - **解释:**
    ///         - 最终的路线用加粗进行了标记。
    ///         - 我们必须等到时间为 16，此时才能保证平台 (0, 0) 和 (4, 4) 是连通的
    /// + **提示:**
    ///     * `n == grid.length`
    ///     * `n == grid[i].length`
    ///     * `1 <= n <= 50`
    ///     * `0 <= grid[i][j] < n2`
    ///     * `grid[i][j]` 中每个值 **均无重复**
    /// + Related Topics
    ///     * 深度优先搜索
    ///     * 广度优先搜索
    ///     * 并查集
    ///     * 数组
    ///     * 二分查找
    ///     * 矩阵
    ///     * 堆（优先队列）
    /// * 👍 217
    /// * 👎 0
    pub fn swim_in_water(grid: Vec<Vec<i32>>) -> i32 {
        const DIRS: [(i32, i32); 4] = [(-1, 0), (1, 0), (0, -1), (0, 1)];
        #[derive(PartialEq, Eq)]
        struct Item(usize, usize, i32);
        impl PartialOrd for Item {
            fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
                other.2.partial_cmp(&self.2)
            }
        }
        impl Ord for Item {
            fn cmp(&self, other: &Self) -> std::cmp::Ordering {
                other.2.cmp(&self.2)
            }
        }
        let n = grid.len();
        let mut queue = std::collections::binary_heap::BinaryHeap::<Item>::new();
        queue.push(Item(0, 0, grid[0][0]));
        let mut dist = vec![vec![(n * n) as i32; n]; n];
        dist[0][0] = grid[0][0];
        let mut visited = vec![vec![false; n]; n];
        while !queue.is_empty() {
            let Item(x, y, _) = queue.pop().unwrap();
            if visited[x][y] { continue; }
            if x == n - 1 && y == n - 1 { return dist[n - 1][n - 1]; }
            visited[x][y] = true;
            for (dx, dy) in DIRS.iter() {
                let (nx, ny) = ((x as i32 + dx) as usize, (y as i32 + dy) as usize);
                if nx < n && ny < n && !visited[nx][ny] && grid[nx][ny].max(dist[x][y]) < dist[nx][ny] {
                    dist[nx][ny] = grid[nx][ny].max(dist[x][y]);
                    queue.push(Item(nx, ny, grid[nx][ny]));
                }
            }
        }
        -1
    }
}

#[cfg(test)]
mod test {
    use crate::Solution;

    #[test]
    fn test_q0778() {
        let grid = vec![
            vec![0, 1, 2, 3, 4],
            vec![24, 23, 22, 21, 5],
            vec![12, 13, 14, 15, 16],
            vec![11, 17, 18, 19, 20],
            vec![10, 9, 8, 7, 6]];
        assert_eq!(Solution::swim_in_water(grid), 16);
    }
}
