use crate::Solution;

impl Solution {
    /// [803.打砖块](https://leetcode-cn.com/problems/bricks-falling-when-hit/)
    ///
    /// 2022-02-21 17:21:08
    ///
    /// 有一个 `m x n` 的二元网格 `grid` ，其中 `1` 表示砖块，`0` 表示空白。砖块 **稳定**（不会掉落）的前提是：
    ///
    /// >   一块砖直接连接到网格的顶部，或者
    /// >   至少有一块相邻（4 个方向之一）砖块 **稳定** 不会掉落时
    ///
    /// 给你一个数组 `hits` ，这是需要依次消除砖块的位置。每当消除 `hits[i] = (rowi, coli)` 位置上的砖块时，对应位置的砖块（若存在）会消失，然后其他的砖块可能因为这一消除操作而 **掉落** 。一旦砖块掉落，它会 **立即** 从网格 `grid` 中消失（即，它不会落在其他稳定的砖块上）。
    ///
    /// 返回一个数组 `result` ，其中 `result[i]` 表示第 `i` 次消除操作对应掉落的砖块数目。
    ///
    /// + **注意**，消除可能指向是没有砖块的空白位置，如果发生这种情况，则没有砖块掉落。
    /// + **示例 1：**
    ///     + **输入：** grid = \[\[1,0,0,0\],\[1,1,1,0\]\], hits = \[\[1,0\]\]
    ///     + **输出：** \[2\]
    ///     + **解释：**
    ///         - 网格开始为：
    ///         - \[\[1,0,0,0\]，
    ///         -  \[**1**,1,1,0\]\]
    ///         - 消除 (1,0) 处加粗的砖块，得到网格：
    ///         - \[\[1,0,0,0\]
    ///         -  \[0,**1**,**1**,0\]\]
    ///         - 两个加粗的砖不再稳定，因为它们不再与顶部相连，也不再与另一个稳定的砖相邻，因此它们将掉落。得到网格：
    ///         - \[\[1,0,0,0\],
    ///         -  \[0,0,0,0\]\]
    ///         - 因此，结果为 \[2\] 。
    ///
    /// + **示例 2：**
    ///
    ///     + **输入：** grid = \[\[1,0,0,0\],\[1,1,0,0\]\], hits = \[\[1,1\],\[1,0\]\]
    ///     + **输出：** \[0,0\]
    ///     + **解释：**
    ///         - 网格开始为：
    ///         - \[\[1,0,0,0\],
    ///         -  \[1,**1**,0,0\]\]
    ///         - 消除 (1,1) 处加粗的砖块，得到网格：
    ///         - \[\[1,0,0,0\],
    ///         -  \[1,0,0,0\]\]
    ///         - 剩下的砖都很稳定，所以不会掉落。网格保持不变：
    ///         - \[\[1,0,0,0\],
    ///         -  \[**1**,0,0,0\]\]
    ///         - 接下来消除 (1,0) 处加粗的砖块，得到网格：
    ///         - \[\[1,0,0,0\],
    ///         -  \[0,0,0,0\]\]
    ///         - 剩下的砖块仍然是稳定的，所以不会有砖块掉落。
    ///         - 因此，结果为 \[0,0\] 。
    ///
    /// + **提示：**
    ///     * `m == grid.length`
    ///     * `n == grid[i].length`
    ///     * `1 <= m, n <= 200`
    ///     * `grid[i][j]` 为 `0` 或 `1`
    ///     * `1 <= hits.length <= 4 * 104`
    ///     * `hits[i].length == 2`
    ///     * `0 <= xi <= m - 1`
    ///     * `0 <= yi <= n - 1`
    ///     * 所有 `(xi, yi)` 互不相同
    /// + Related Topics
    ///     * 并查集
    ///     * 数组
    ///     * 矩阵
    /// * 👍 226
    /// * 👎 0
    pub fn hit_bricks(grid: Vec<Vec<i32>>, hits: Vec<Vec<i32>>) -> Vec<i32> {
        const DIRECTIONS: [[i32; 2]; 4] = [[0, 1], [1, 0], [-1, 0], [0, -1]];
        let rows = grid.len();
        let cols = grid[0].len();
        let mut copy = grid.clone();
        for hit in &hits {
            copy[hit[0] as usize][hit[1] as usize] = 0;
        }
        let grid_len = rows * cols;

        let mut parent = (0..(grid_len + 1)).collect::<Vec<usize>>();
        let mut size = vec![1usize; grid_len + 1];

        fn in_area(x: usize, y: usize, rows: usize, cols: usize) -> bool {
            x < rows && y < cols
        }
        fn get_idx(x: usize, y: usize, cols: usize) -> usize {
            x * cols + y
        }
        fn find(parent: &mut Vec<usize>, x: usize) -> usize {
            if x != parent[x] {
                parent[x] = find(parent, parent[x]);
            }
            parent[x]
        }
        fn union(parent: &mut Vec<usize>, size: &mut Vec<usize>, x: usize, y: usize) {
            let root_x = find(parent, x);
            let root_y = find(parent, y);
            if root_x == root_y {
                return;
            }
            parent[root_x] = root_y;
            size[root_y] += size[root_x];
        }
        fn get_size(parent: &mut Vec<usize>, size: &mut Vec<usize>, x: usize) -> usize {
            return size[find(parent, x)];
        }

        for j in 0..cols {
            if copy[0][j] == 1 {
                union(&mut parent, &mut size, j, grid_len);
            }
        }

        for i in 1..rows {
            for j in 0..cols {
                if copy[i][j] == 1 {
                    if copy[i - 1][j] == 1 {
                        union(&mut parent, &mut size, get_idx(i - 1, j, cols), get_idx(i, j, cols));
                    }
                    if j > 0 && copy[i][j - 1] == 1 {
                        union(&mut parent, &mut size, get_idx(i, j - 1, cols), get_idx(i, j, cols));
                    }
                }
            }
        }
        let mut res = vec![0i32; hits.len()];
        for i in (0..res.len()).rev() {
            let x = hits[i][0] as usize;
            let y = hits[i][1] as usize;
            if grid[x][y] == 0 {
                continue;
            }
            let origin = get_size(&mut parent, &mut size, grid_len);

            if x == 0 {
                union(&mut parent, &mut size, y, grid_len);
            }

            for direction in &DIRECTIONS {
                let new_x = (x as i32 + direction[0]) as usize;
                let new_y = (y as i32 + direction[1]) as usize;
                if in_area(new_x, new_y, rows, cols) && copy[new_x][new_y] == 1 {
                    union(&mut parent, &mut size, get_idx(x, y, cols), get_idx(new_x, new_y, cols));
                }
            }

            let current = get_size(&mut parent, &mut size, grid_len);
            res[i] = if current < origin + 1 {
                0
            } else {
                (current - origin - 1) as i32
            };
            copy[x][y] = 1;
        }
        res
    }
}

#[cfg(test)]
mod test {
    use crate::Solution;

    #[test]
    fn test_q0803() {
        let grid = vec![vec![1, 0, 0, 0], vec![1, 1, 1, 0]];
        let hits = vec![vec![1, 0]];
        let vec1 = Solution::hit_bricks(grid, hits);
        //println!("{:?}", vec1);
        assert_eq!(vec1, vec![2]);
    }
}
