use crate::Solution;

impl Solution {
    /// [959.由斜杠划分区域](https://leetcode-cn.com/problems/regions-cut-by-slashes/)
    ///
    /// 2022-02-21 17:47:06
    ///
    /// 在由 `1 x 1` 方格组成的 `n x n` 网格 `grid` 中，每个 `1 x 1` 方块由 `'/'`、`'\'` 或空格构成。这些字符会将方块划分为一些共边的区域。
    ///
    /// 给定网格 `grid` 表示为一个字符串数组，返回 _区域的数量_ 。
    ///
    /// 请注意，反斜杠字符是转义的，因此 `'\'` 用 `'\\'` 表示。
    ///
    /// + **示例 1：**
    ///     + ![](https://assets.leetcode.com/uploads/2018/12/15/1.png)
    ///     + **输入：** grid = \[" /","/ "\]
    ///     + **输出：** 2
    /// + **示例 2：**
    ///     + ![](https://assets.leetcode.com/uploads/2018/12/15/2.png)
    ///     + **输入：** grid = \[" /","  "\]
    ///     + **输出：** 1
    /// + **示例 3：**
    ///     + ![](https://assets.leetcode.com/uploads/2018/12/15/4.png)
    ///     + **输入：** grid = \["/\\\\","\\\\/"\]
    ///     + **输出：** 5
    ///     + **解释：** 回想一下，因为 \\ 字符是转义的，所以 "/\\\\" 表示 /\\，而 "\\\\/" 表示 \\/。
    /// + **提示：**
    ///     * `n == grid.length == grid[i].length`
    ///     * `1 <= n <= 30`
    ///     * `grid[i][j]` 是 `'/'`、`'\'`、或 `' '`
    /// + Related Topics
    ///     * 深度优先搜索
    ///     * 广度优先搜索
    ///     * 并查集
    ///     * 图
    /// * 👍 298
    /// * 👎 0
    pub fn regions_by_slashes(grid: Vec<String>) -> i32 {
        fn find(parent: &mut Vec<usize>, x: usize) -> usize {
            if parent[x] == x {
                x
            } else {
                parent[x] = find(parent, parent[x]);
                parent[x]
            }
        }
        fn union(parent: &mut Vec<usize>, x: usize, y: usize) {
            let x_root = find(parent, x);
            let y_root = find(parent, y);
            parent[x_root] = y_root;
        }
        let n = grid.len();
        let mut parent = (0..(n * n * 4)).collect::<Vec<usize>>();
        for (i, s) in grid.into_iter().enumerate() {
            let s = s.into_bytes();
            for j in 0..n {
                let idx = i * n + j;
                if i < n - 1 {
                    let bottom = idx + n;
                    union(&mut parent, idx * 4 + 2, bottom * 4);
                }
                if j < n - 1 {
                    let right = idx + 1;
                    union(&mut parent, idx * 4 + 1, right * 4 + 3);
                }
                if s[j] == b'/' {
                    union(&mut parent, idx * 4, idx * 4 + 3);
                    union(&mut parent, idx * 4 + 1, idx * 4 + 2);
                } else if s[j] == b'\\' {
                    union(&mut parent, idx * 4, idx * 4 + 1);
                    union(&mut parent, idx * 4 + 2, idx * 4 + 3);
                } else {
                    union(&mut parent, idx * 4, idx * 4 + 1);
                    union(&mut parent, idx * 4 + 1, idx * 4 + 2);
                    union(&mut parent, idx * 4 + 2, idx * 4 + 3);
                }
            }
        }
        let mut fathers = std::collections::HashSet::<usize>::new();
        for i in 0..(n * n * 4) {
            let fa = find(&mut parent, i);
            fathers.insert(fa);
        }
        fathers.len() as i32
    }
}

#[cfg(test)]
mod test {
    use crate::Solution;

    #[test]
    fn test_q0959() {
        assert_eq!(Solution::regions_by_slashes(vec!["/\\".to_string(), "\\/".to_string()]), 5);
    }
}
