//! 此模块定义数据结构项目架构中用到的标签

use std::rc::Rc;
use std::cell::RefCell;

/// 单链表
#[derive(PartialEq, Eq, Clone, Debug)]
pub struct ListNode {
    pub val: i32,
    pub next: Option<Box<ListNode>>,
}

impl ListNode {
    #[inline]
    pub(crate) fn new(val: i32) -> Self {
        ListNode {
            next: None,
            val,
        }
    }
}

#[cfg(test)]
mod test {
    use crate::structure::ListNode;

    impl ListNode {
        pub(crate) fn from_vec(vec: Vec<i32>) -> Option<Box<ListNode>> {
            let mut node = None;
            for val in vec.into_iter().rev() {
                node = Some(Box::new(ListNode { val, next: node }));
            }
            node
        }

        //noinspection RsSelfConvention
        pub(crate) fn to_vec(mut list: Option<Box<ListNode>>) -> Vec<i32> {
            let mut ret = vec![];
            while let Some(n) = list {
                ret.push(n.val);
                list = n.next;
            }
            ret
        }
    }
}

// Definition for a binary tree node.
#[derive(Debug, PartialEq, Eq)]
pub struct TreeNode {
    pub val: i32,
    pub left: Option<Rc<RefCell<TreeNode>>>,
    pub right: Option<Rc<RefCell<TreeNode>>>,
}

impl TreeNode {
    #[inline]
    pub fn new(val: i32) -> Self {
        TreeNode {
            val,
            left: None,
            right: None,
        }
    }
}